/*
+-----------------------------------------------------------------------+
|                                                                       |
|           DICO.C R.Cougnenc 1992 - Domaine Public                     |
|           ---------------------------------------                     |
|                                                                       |
|       Le dictionnaire est organise en 26 fichiers, un                 |
|       pour chaque lettre. Ceci constitue un petit arbre               |
|       largement suffisant.                                            |
|                                                                       |
|       Si le mot commence par une lettre, on recherche                 |
|       dans le fichier correspondant, a la maniere de                  |
|       grep un peu.                                                    |
|                                                                       |
|       S'il commence par une expression reguliere, on                  |
|       recherche dans les 26 fichiers en totalite.                     |
|       C'est toutefois d'une vitesse supportable.                      |
+-----------------------------------------------------------------------+
| Portage Windows par Serge Delbono                                     |
| Ce module est uns "sous-programmisation" de la version DOS/Linux      |
| Seul le dictionnaire ISO est supporte                                 |
+-----------------------------------------------------------------------+
*/
#define MYOS "WINDOWS"            /* Pour affichage version only    */
#include "wdic.h"

#define IBM_TERM    1                   /* Pour translation affichage      */
#define ASCII_TERM  2
#define ISO_TERM    3
#define COMMENT_CHAR 35                 /* '#' ligne de commentaire        */

#define MYVBUF 10240                     /* pour setvbuf() eventuel         */

typedef unsigned char byte;

                                  /* protos fonctions speciales accents... */
byte ToLower( byte c );
byte ToUpper( byte c );
// byte *StrUpr ( byte *str);
#define StrUpr AnsiUpper
// byte *StrLwr ( byte *str);
#define StrLwr AnsiLower
void StrAscii( byte *str);
__inline byte equival( byte c  );
void nolf (char *str) ;
int fmatch(char *_fname, char *_pat);
match(char *_fname, char *_pat) ;
BOOL TraiteTab(char *buf) ;
__inline int isjoker( int c ) ;
void CleanName (byte *str) ;

/*------------------------------------------------------------------------*/
/*             Ici seul endroit pour modifier le numero de version:       */
/*                                                                        */
            char *__Version__ ="1.02" ;
/*                                                                        */
/*------------------------------------------------------------------------*/

int myterm = 0     ;    /* translation terminal pour accents */

/*--------------------------------------------------------------------------*/
/*
 *  lookfor: Recherche dans le fichier de l'initiale correspondante
 *           de l'expression pattern.
 */

void lookfor ( char *pattern, // Pattern a chercher
          HWND hlist)    // Handle fenetre list box
{
 FILE *fp ;
 char fname[127];
 char buf[80];
 char sbuf[80];
 byte car ;

    NbItem = 0 ;
    StrLwr( pattern );  /* passe en minuscules comme le dico */
    StrAscii( pattern);          /* Passe en non accent-sensitive */
    car = equival (pattern[0]);

    sprintf(fname, "%s%c%s",PathDic, car, DICT_EXT );
    AnsiToOem (fname, fname) ;

    if(( fp = fopen(fname,"r")) == NULL ) {
        Fatal ("Dictionnaire non trouv !") ;
        return ;
    }

#ifdef MUST_HAVE_BUF
    setvbuf( fp, NULL, _IOFBF, MYVBUF );
#endif

    while( fgets(buf,79,fp) )
    {
        if( *buf == '#' )        /* Ligne de commentaire          */
            continue ;
        nolf( buf );             /* supprime le line-feed */
        strcpy( sbuf, buf );     /* No accent sensitive   */
        StrLwr (sbuf) ;       /* SD : sinon trouve pas si majuscules */
                                /* dans le dico ! */
        CleanName (sbuf) ;

        if( fmatch(sbuf,pattern) )
          if (Trouve (buf,hlist)) return ; // retour si liste pleine
    }

    fclose( fp) ;
    return ;
}
/*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*/
/*
 *  lookall:  Recherche dans TOUS les 26 fichiers de l'expression pattern.
 *            Srement 26 fois plus lent que l'autre pour la lettre z :-)
 *
 */
void lookall ( char *pattern, HWND hlist)
{
 FILE *fp ;
 char fname[127];
 char buf[80];
 char sbuf[80];
 byte letter;

    NbItem = 0 ;
    StrLwr( pattern ); /* passe en minuscules comme le dico */

    StrAscii( pattern);          /* Passe en non accent-sensitive */

    for( letter = 'a' ; letter <= 'z' ; letter++ )
    {
        sprintf(fname, "%s%c%s",PathDic, letter, DICT_EXT );
        AnsiToOem (fname, fname) ;
        if(( fp = fopen(fname,"r")) == NULL )
        {
            Fatal ("Dictionnaire non trouv !") ;
            return ;
        }

#ifdef MUST_HAVE_BUF
        setvbuf( fp, NULL, _IOFBF, MYVBUF );
#endif

        while( fgets(buf,79,fp) )
        {
            if( *buf == '#' )        /* Ligne de commentaire          */
                continue ;
            nolf( buf );            /* supprime le line-feed */
            strcpy( sbuf, buf);     /* no accent sensitive  */
            StrLwr (sbuf) ;       /* SD : sinon trouve pas si majuscules */
                                    /* dans le dico ! */
            CleanName (sbuf) ;
            if( fmatch(sbuf,pattern) )
                 if (Trouve (buf,hlist)) return ; // retour si liste pleine
        }
        fclose( fp );
        GotoBistrot() ;
    }
    return ;
}
/*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*/
/*
 *   is joker: Retourne 0 si le caractere n'est pas un truc pour
 *            un debut d'expressions regulieres.
 *
 */
__inline int isjoker( int c )
{
    switch( c )
    {
        case '[':
        case ']':
        case '?':
        case '*':
        case '^':
        case '-':
            return TRUE ;
    }

    return FALSE  ;
}
/*--------------------------------------------------------------------------*/



/*--------------------------------------------------------------------------*/
/*
 * nolf: Supprime un line feed a la fin d'une chaine.
 *
 */
void nolf( char *str)
{
    while( *str )
    {
      if( *str == '\n')
        *str = '\0' ;
      str++;
    }
}
/*--------------------------------------------------------------------------*/



/*--------------------------------------------------------------------------*/
/*
    Fonction : int fmatch(char *string,char *pattern);

    Gestion simplifiee des expressions regulieres, afin d'etre simple
    et compatible MS-DOS ou autres systemes.
*/
                                                /* d'apres freeware qnx */


#define MATCH   1

int fmatch(char *_fname, char *_pat)
{
    register char *pat, *fname;

    pat = _pat;
    fname = _fname;

    if(*pat++ == '*') {
        if(*pat == '\0')
            return(MATCH);

        while(*fname)
            if(match(fname++, pat))
                return(MATCH);

        return(0);
    }
    else
        return(match(fname, pat + -1));
}



match(char *_fname, char *_pat)
{
    register char *pat, *fname;
    char not_flag, found_flag;

    pat = _pat;
    fname = _fname;

    while(*pat && *fname) {
        switch(*pat) {

        case '?':
            break;

        case '[':
            not_flag = (*++pat == '^')||(*pat == '!') ;
            found_flag = 0;

            while(*pat  &&  *pat++ != ']')
            {
                if(*(pat-1) == *fname)
                    found_flag = 1;
/* SD : traitement des [a-z] */
                 else if (*pat =='-'){
                    char ch, chorg ;
                    int incr ;
                    chorg = *(pat-1) ;
                    pat++ ;
                    if (*pat){
                      incr = (*pat > chorg) ? 1 : -1 ;
                      for (ch=chorg;ch!= *pat + incr ;ch+= incr ){
                         if (ch==*fname) {
                           found_flag = 1 ; break ;
                         }
                       } /* fin for ch */
                       pat++ ;
                    } /* fin if *pat */
                 } /* fin if */

            }

            if(found_flag == not_flag)
                return(0);

            --pat;
            break;

        case '*':
            return(fmatch(fname, pat));

        default:
            if(*pat != *fname)
                return(0);
        }

        ++pat;
        ++fname;
    }

 return((*pat == 0 || (*pat == '*' && *++pat == 0))
         &&  *fname == 0 ? MATCH : 0);
}

/*----------------------( fin de fmatch() )--------------------------------*/


/*--------------------------------------------------------------------------*/
/*
 *    ToLower: special tolower 8bits, don't touch 8 bit chars.
 *
 *
 */
 // Remplace par AnsiLower de ouindoz
/*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*/
/*
 *  StrLwr:  Special strlwr routine, works whith 8 bit chars ISO/IBM,
 *           do not modify these characters.
 *
 */
 // Remplace par AnsiLower de ouindoz

/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/*
 *    ToUpper: special toupper 8bits, don't touch 8 bit chars.
 *
 *
 */
byte ToUpper (byte c )
{
  byte ch [2] ;
  ch[0] = c ;
  ch[1] = '\0' ;
  AnsiUpper(ch) ; // bof !
  return ch[0] ;
}
/*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*/
/*
 *  StrUpr:  Special strupr routine, works whith 8 bit chars ISO/IBM,
 *           do not modify these characters.
 *
 */
// ANSIUPPER

/*--------------------------------------------------------------------------*/

/****************************************************************************/
/********************   Fonctions dependantes du jeu de cars utilise ********/
/****************************************************************************/


/*--------------------------------------------------------------------------*/
/*
 *  equival( ch ) :   Renvoie le caractere non accentue equivalent a une
 *            lettre accentuee minuscule, en Francais.
 */

__inline byte equival( byte ch )
{
#ifdef OEM_CHAR
// Jeux IBM-PC
static char traduc[] = {
      /*     0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F */
    /* 80 */'C','u','e','a','a','a','a','c','e','e','e','i','i','i','A','A',
    /* 90 */'E','a','A','o','o','o','u','u','y','O','U','c','L','Y','P','f',
    /* A0 */'a','i','o','u','n','N','a','o','?','+','+','X','X','!','<','>',
    /* B0 */'X','X','X','|','+','+','+','+','+','+','|','+','+','+','+','+',
    /* C0 */'+','+','+','+','-','+','+','+','+','+','+','+','+','-','+','+',
    /* D0 */'+','+','+','+','+','+','+','+','+','+','+','X','_','|','|','-',
    /* E0 */'a','b','G','P','E','o','u','t','o','O','O','d','8','o','E','A',
    /* F0 */'=','+','>','<','f','f','/','=','.','.','.','V','n','2','*',' '
} ;
#else
static char traduc[] = {
      /*     0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F */
    /* 80 */'.','.','.','.','.','.','.','.','.','.','.','.','.','.','.','.',
    /* 90 */'.','`',39 ,'"','"','.','.','.','.','.','.','.','.','.','.','.',
    /* A0 */' ','!','c','L','o','Y','|','.','.','C','a','"','+','-','R','-',
    /* B0 */'o','+','2','3',39 ,'u','.','.',',','1','o','"','1','1','3','?',
    /* C0 */'A','A','A','A','A','A','A','C','E','E','E','E','I','I','I','I',
    /* D0 */'D','N','O','O','O','O','O','_','0','U','U','U','U','Y','B','b',
    /* E0 */'a','a','a','a','a','a','a','c','e','e','e','e','i','i','i','i',
    /* F0 */'d','n','o','o','o','o','o','o','o','u','u','u','u','y','b','y'
} ;
#endif
        if (ch & 0x80){
          ch &=0x7F ;
          return (traduc [ch]) ;
        } else {
          return ch ;
        } /* fin if */
}
/*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*/
/*
 * StrAscii:     Supprime les accents Francais dans une chaine, la rendant
 *               pur 7 bits.
 *               Vire galement les espaces / tabulation
 *
 *               Retourne un pointeur sur la chaine modifiee.
 */
void
StrAscii ( byte *str)
{
  byte ch ;
  char *p = str ;
  while (ch = *str) {
        ch = equival(ch) ;
        if ( ! isspace(ch)) {
                *p = ch ;
                p++ ;
        } /* fin if */
        str++ ;
  } /* fin while */
  *p='\0' ;

 return ;
}

/*********************************************************/
/*                                                       */
/*   Supprime tout ce qui peut gener dans la recherche   */
/*                (tiret, espace etc...)                 */
/*                                                       */
/*********************************************************/

void CleanName (byte *str)
{
  byte ch ;
  char *p = str ;
  while (ch = *str) {
        ch = equival(ch) ;
        if ( IsCharAlphaNumeric (ch)) {
                *p = ch ;
                p++ ;
        } /* fin if */
        str++ ;
  } /* fin while */
  *p='\0' ;
 return ;
}

/*--------------------------------------------------------------------------*/

/***********************************************************/
/*                                                         */
/*                     Pour Windows :                      */
/*       On remplace les tabulations par plus joli !       */
/* et puis on y met tout Zoli                              */
/*                                                         */
/***********************************************************/
/* retour : y'a-t-il eu tabulation ? */
BOOL TraiteTab(char *buf)
{
  BOOL maj ;
  char c ;
  BOOL retour=FALSE ;
  buf[0] = ToUpper (buf[0]) ;
  maj=TRUE ;

  while ((c=*buf)!='\0'){
    if (maj) {
      *buf = ToUpper (c) ;
      maj=FALSE ;
    } else
    {
      if (c=='\t') {
         *buf = '*' ;
         retour = TRUE ;
      }
      if (! IsCharAlphaNumeric (c))
        maj=TRUE ;
    }
    buf ++ ;
  } /* fin while */
  return retour ;
}

/***************************************************************/
/*                           OinDoz                            */
/*   Affichage du mot Trouve dans la list box de la fenetre    */
/*                         Principale                          */
/***************************************************************/

BOOL Trouve (char *buf, HWND hlist)
{
    if (TraiteTab (buf))
    {
        if (!bAutoCP){  // code postaux interdits
          return FALSE ;
        } /* fin if */
    } else {
        if (!bAutoVoc){
          return FALSE ; // vocab interdit
        } /* fin if */

    }
    if (bListFull)
      return TRUE ;
// OINDOZES !!!!
// On envoie le mot trouve a qui de droit (ici, la list-box
// ou est affiche le resultat)
    SendMessage (hlist, LB_ADDSTRING, 0, (LONG) (LPSTR) buf) ;
    NbItem ++ ;
// On affiche les stats...
    if (NbItem == 1){
      SetDlgItemText (hWndMain, MAIN_MOTS,"1 mot trouv") ;
    } else {
      char pText [30] ;
      sprintf (pText, "%ld mots trouvs",NbItem) ;
      SetDlgItemText (hWndMain, MAIN_MOTS,pText) ;
    } /* fin if */

// OinDoz n'etant pas "Multitache preensif", il faut penser aux
// copain
   GotoBistrot() ;
   return FALSE ;
}
